function [X, ifilenames, feature_names, features, names] = evaluateFeatures(instancepath)

tmpl = dir(instancepath{1});
filelist = tmpl(3:end);
for i = 2:length(instancepath)
    tmpl = dir(instancepath{i});
    filelist = [filelist;tmpl(3:end)];
end

warning('off','MATLAB:polyfit:RepeatedPointsOrRescale')

featlist = Knapsack.featureList;
redfeatlist = Knapsack.reducedFeatureList;
KP_FEATURES = size(featlist, 1);
RK_FEATURES = size(redfeatlist, 1);
NO_FEATURES = KP_FEATURES + RK_FEATURES;
NO_INSTANCES = (length(filelist));

if true
    ifilenames = cell(NO_INSTANCES,1);
    for i = 1:NO_INSTANCES
        ifilenames{i} = filelist(i).name;
    end
end

if true
    features = zeros(NO_INSTANCES, NO_FEATURES);
    oob = zeros(NO_INSTANCES, NO_FEATURES);
    names = cell(NO_INSTANCES, 2);
    feature_names = cell(NO_FEATURES,1);
    isTrivial = zeros(NO_INSTANCES, 1);
    isError = zeros(NO_INSTANCES, 1);
    featureBounds = zeros(2, NO_FEATURES);
end


%Feature generating and names listing loop
for i = 1:NO_INSTANCES
    fprintf("Current instance number: %d of %d\n", i, NO_INSTANCES);
    
    KP = Knapsack.readFromFile(ifilenames{i}, [filelist(i).folder '\']);%manually change directory here as well if needed
    
    %fprintf("Finished reading file\n");
    
    for j = 1:KP_FEATURES
        features(i,j) = featlist{j,2}(KP);
        feature_names{j,1} = featlist{j,1};
        if features(i,j) < featlist{j,3}
            oob(i,j) = 1;
        end
        if features(i,j) > featlist{j,4}
            oob(i,j) = 1;
        end
        %fprintf("Computed feature %d\n", j);
    end
    
    reductions = [];
    RK = KP.reducedKnapsack;
    MK = KP;
    if RK.Capacity < 0
        error("Reduced knapsack invalid\n")
    end
    reductions = KP.Size - RK.Size;
    
    %fprintf("Reduced knapsack\n", j);
    
    for j = 1:RK_FEATURES
        features(i,j+KP_FEATURES) = redfeatlist{j,2}(RK);
        feature_names{j+KP_FEATURES,1} = char("Reduced" + redfeatlist{j,1});
        if features(i,j+KP_FEATURES) < redfeatlist{j,3}
            oob(i,j+KP_FEATURES) = 1;
        end
        if features(i,j+KP_FEATURES) > redfeatlist{j,4}
            oob(i,j+KP_FEATURES) = 1;
        end
        %fprintf("Computed feature %d\n", j+KP_FEATURES);
    end
    
    MK = RK.reducedKnapsack;
    if MK.Size ~= RK.Size
        reductions = [reductions; MK.Size - RK.Size]; %#ok<AGROW>
        cursize = MK.Size;
        MK = MK.reducedKnapsack;
        while cursize ~= MK.Size
            reductions = [reductions; cursize ~= MK.Size]; %#ok<AGROW>
            cursize = MK.Size;
            MK = MK.reducedKnapsack;
        end
    end
    
    names(i,1) = {KP.Name};
    names(i,2) = {KP.Class};
end

z = cell(NO_INSTANCES,1);
names = [names, z];

for i = 1:NO_INSTANCES
    if strcmpi(names{i,1}(1:7),'GAUnstr') == 1 
        names{i,2} = 'GA_Unstr';
    elseif strcmpi(names{i,1}(1:6),'GACubc') == 1 
        names{i,2} = 'GA_Cubc';
    elseif strcmpi(names{i,1}(1:7),'GAMulti') == 1 
        names{i,2} = 'GA_Multi';
    elseif strcmpi(names{i,1}(1:7),'GAProft') == 1 
        names{i,2} = 'GA_Proft';
    elseif strcmpi(names{i,1}(1:7),'GASpanr') == 1 
        names{i,2} = 'GA_Spanr';
    end
end

for i = 1:NO_INSTANCES
    if strcmpi(names(i,2),'5KP') == 1
        names{i,3} = 24;
    elseif strcmpi(names(i,2),'AlmostStronglyCorrelated') == 1
        names{i,3} = 6;
    elseif strcmpi(names(i,2),'AvisKnapsack') == 1
        names{i,3} = 19;
    elseif strcmpi(names(i,2),'AvisSubsetSum') == 1
        names{i,3} = 20;
    elseif strcmpi(names(i,2),'Circle') == 1
        names{i,3} = 16;
    elseif strcmpi(names(i,2),'CubeFit') == 1
        names{i,3} = 22;
    elseif strcmpi(names(i,2),'EvenOddStronglyCorrelated') == 1
        names{i,3} = 8;
    elseif strcmpi(names(i,2),'EvenOddSubsetSum') == 1
        names{i,3} = 9;
    elseif strcmpi(names(i,2),'HillReilly') == 1
        names{i,3} = 23;
    elseif strcmpi(names(i,2),'InvStrongCorrelated') == 1
        names{i,3} = 5;
    elseif strcmpi(names(i,2),'Multiple') == 1
        names{i,3} = 14;
    elseif strcmpi(names(i,2),'NoSmallWeights') == 1
        names{i,3} = 10;
    elseif strcmpi(names(i,2),'ProfitCeiling') == 1
        names{i,3} = 15;
    elseif strcmpi(names(i,2),'QuadFit') == 1
        names{i,3} = 21;
    elseif strcmpi(names(i,2),'SimilarWeightsUncorrelated') == 1
        names{i,3} = 7;
    elseif strcmpi(names(i,2),'SpannerUncorrelated') == 1
        names{i,3} = 11;
    elseif strcmpi(names(i,2),'SpannerWeak') == 1
        names{i,3} = 12;
    elseif strcmpi(names(i,2),'SpannerStrong') == 1
        names{i,3} = 13;
    elseif strcmpi(names(i,2),'StronglyCorrelated') == 1
        names{i,3} = 3;
    elseif strcmpi(names(i,2),'SubsetSum') == 1
        names{i,3} = 4;
    elseif strcmpi(names(i,2),'TwoGroups') == 1
        names{i,3} = 17;
    elseif strcmpi(names(i,2),'Uncorrelated') == 1
        names{i,3} = 1;
    elseif strcmpi(names(i,2),'WeaklyCorrelated') == 1
        names{i,3} = 2;
    elseif strcmpi(names(i,2),'WeaklyCorrelatedLO') == 1
        names{i,3} = 18;
    end
end

% Set up MATILDA input
X2 = [feature_names';num2cell(features)];
X = X2(:,:);

%save('evalfeatures-output')

end